# **An implementation of ROOT's TDataSource interface that reads xAOD files.**

This repository currently contains three packages.  
**1. xAODdataSource:** A collection of interfaces and classes which includes
implementation of ROOT's TDataSource interface and supporting classes which
interact with ATLAS event store.  
**2. EventLoopTests:** A set of test applications, to demonstrate use cases of
ATLAS EventLoop tool, as used for accessing ATLAS EDM objects using AnaAlgorithm
framework. [https://atlassoftwaredocs.web.cern.ch/ABtutorial/]  
**3. AppUtils**: A set of interfaces and utility classes to support test
applications used by above two packages.

Each package has directory structure as below:  
**PackageName:** Interfaces, public headers of package.  
**Root:** Implementation, source files.  
**Test:** Unit tests and Integration tests.  
**Apps:** Standalone test applications.  

## **Setup instructions**  
**Build/Install ROOT and prepare environment** [Tested with cxx14=ON]   
source ${ROOTSYS}/bin/thisroot.sh

**Setup athena repository**  
mkdir AnalysisBase, cd AnalysisBase  
git clone https://:@gitlab.cern.ch:8443/[user_name]/athena.git  
cd athena  
git remote add upstream https://:@gitlab.cern.ch:8443/atlas/athena.git  
git fetch upstream  
git checkout release/21.2.xx        //use 21.2 for latest version. More: [https://atlassoftwaredocs.web.cern.ch/ABtutorial/release_setup/#building-your-own-release-optional--advanced]  

**Note:** Current AnalysisBase build scripts force install ROOT from AnalysisBase  
release if any of these external packages are built [XRootD, DCAP, LibXML2]  
For linking AnalysisBase with development version of ROOT, currently a workaround  
is used by modifying /athena/Build/AtlasBuildScripts/build_atlasexternals.sh with  
-DATLAS_BUILD_XROOTD:BOOL=FALSE -DATLAS_BUILD_DCAP:BOOL=FALSE -DATLAS_BUILD_LIBXML2:BOOL=FALSE as cmake parameter.  

**Build AnalysisBase** [Ubuntu 16.04.LTS]
export MAKEFLAGS=-jx  
./athena/Projects/AnalysisBase/build_externals.sh -c  
./athena/Projects/AnalysisBase/build.sh -acmi  
source AnalysisBase/build/install/AnalysisBase/21.2.31/InstallArea/x86_64-ubuntu1604-gcc54-opt/setup.sh

**Setup AnalysisBase** [CentOS7]        //Needs cvmfs access and basic ATLAS environment ${ATLAS_LOCAL_ROOT_BASE} More: [https://atlassoftwaredocs.web.cern.ch/ABtutorial/release_setup/]  
cd AnalysisBase  
setupATLAS  
asetup none,gcc62 --cmakesetup  
export MAKEFLAGS=-jx  
./athena/Projects/AnalysisBase/build_externals.sh -c        //This should create the build directory.  
./athena/Projects/AnalysisBase/build.sh -acmi  
asetup AnalysisBase, 21.2.xx --releasesarea=/home/.../AnalysisBase/build/install/

**Setup xAODdataSource**  
mkdir xAODdataSource, cd xAODdataSource  
Clone the repository and checkout suitable tag  
git checkout -b 0.1.xx 0.1.xx  
mkdir build, cd build  
cmake ../xaod-ds  
cmake --build . -- -jx  
source ./x86_64-ubuntu1604-gcc54-opt/setup.sh        //For CentOS7 source ./x86_64-centos7-gcc62-opt/setup.sh

**Test if everything went well**  
mkdir ../run, cd ../run
export DATA=[base_directory_of_xaod_data_files]  
UnitTests.xAODdataSource  
IntTests.xAODdataSource  

**A sample test run of xDataFrame test appication**  
ReadAuxElement.DataFrame --input-files "$DATA/data17_13TeV.00328263.physics_Main.deriv.DAOD_HIGG2D2.f836_m1824_p3213/*"  
xAOD::Init                INFO    Environment initialised for data access  
xAODdataSource::Initialize Distributed [353793] events to [4] slots  
xAODdataSource::Initialize Appended remainder to last Range  
AsgTools_UserCode        INFO    EventInfo.eventNumber Range: [346749618, 2378205328]  
AsgTools_UserCode        INFO    Histogram EventInfo.runNumber:  
AsgTools_UserCode        INFO    Total entries: 353793  
AsgTools_UserCode        INFO    Range: [0, 353793]  
AsgTools_UserCode        INFO    Mean: 328263 StdDev: 0.8739181369878131

**And corresponding test run of EventLoop test appication**  
ReadAuxElement.EventLoop --input-files "$DATA/data17_13TeV.00328263.physics_Main.deriv.DAOD_HIGG2D2.f836_m1824_p3213/*" --out-dir out  
xAOD::Init                INFO    Environment initialised for data access  
Running sample: data17_13TeV.00328263.physics_Main.deriv.DAOD_HIGG2D2.f836_m1824_p3213  
Processing File: file:///Ariane/Data/data17_13TeV.00328263.physics_Main.deriv.DAOD_HIGG2D2.f836_m1824_p3213/DAOD_HIGG2D2.11597434._000104.pool.root.1  
Processing File: file:///Ariane/Data/data17_13TeV.00328263.physics_Main.deriv.DAOD_HIGG2D2.f836_m1824_p3213/DAOD_HIGG2D2.11597434._000468.pool.root.1  
Processed 10000 events  
...  
Processed 340000 events  
Processing File: file:///Ariane/Data/data17_13TeV.00328263.physics_Main.deriv.DAOD_HIGG2D2.f836_m1824_p3213/DAOD_HIGG2D2.11597434._000520.pool.root.1  
Processed 350000 events  
ReadAuxElementAnaAlgor...INFO    EventNumber Range: [346749618, 2378205328]  

worker finished:  
Real time 0:00:05, CP time 5.510  
AsgTools_UserCode        INFO    Histogram EventInfo.runNumber:  
AsgTools_UserCode        INFO    Total entries: 353793  
AsgTools_UserCode        INFO    Range: [0, 353793]  
AsgTools_UserCode        INFO    Mean: 328263 StdDev: 0.8739181369878131  
xAOD::TFileAccessTracer   INFO    Sending file access statistics to http://rucio-lb-prod.cern.ch:18762/traces/  

**There is more...**  
ReadAuxElement.EventLoop --help    
Usage: ReadAuxElement.EventLoop [--option1 value1, ..., --optionN valueN]  
Command line options:  
  -h [ --help ]                         Provide usage help  
  -i [ --input-files ] arg (=\*.root\*) Input filename pattern  
  -o [ --out-dir ] arg (=outDir)        Output directory  
  -m [ --enable-MT ] arg (=1)           [true/false, yes/no] Allows to disable ImpliciteMT feature Default: true  
                                        When disabled, --threads option is ignored  
  -t [ --threads ] arg (=0)             Limit number of threads used for ImpliciteMT feature  
                                        Default: Use all available threads  
  -e [ --max-events ] arg (=18446744073709551615)  
                                        Limit the number of events to process  
                                        Default: Process all events  

**Note:** Most of these options also work with xxx.DataFrame versions except for --max-events.  
Options those are not applicable [e.g. --enable-MT in serial EventLoop], are currently ignored silently. 