#include "xAODdataSource/TxAODdataSource.h"
#include "AppUtils/TOptions.h"
#include "xAODRootAccess/Init.h"
#include "xAODBase/IParticleContainer.h"
#include "AsgTools/MessageCheck.h"
#include <iomanip>


using TDataSource = ROOT::Experimental::TDF::TDataSource;

int main(int argc, char* argv[]) {
    using namespace asg::msgUserCode;
    ANA_CHECK_SET_TYPE(int);   // Required since the return type is different than standard ATLAS StatusCode.

    AppUtils::TOptions options(argc, argv);
    options.interprete();
    if (options.isHelpRequested()) return 0;
    if (options.enableMT()) { ROOT::EnableImplicitMT(options.NThreads()); }

    ANA_CHECK(xAOD::Init()); // Set up the job for xAOD access.

    auto xdf = xAOD::DataSource::make_xAODdataFrame(options.fileNamePattern(), AppUtils::DEFAULT_TREE_NAME);
    auto totalEvents = xdf.Count();

    //Define a limiting function for pt variable.
    constexpr auto ptThreshold = 10000.0;
    const auto ptCut = [](const xAOD::IParticleContainer* particles) {
        std::vector<double> ptVector{};
        for (const auto& p: *particles) {
            if (p->pt() >= ptThreshold) { ptVector.push_back(p->pt()); };
        }
        return ptVector;
    };

    //Define a transient column to hold filtered data.
    constexpr auto ptCutColumn = "ptCutVector";
    auto ptCutData = xdf.Define(ptCutColumn, ptCut, {"Electrons"});

    //Define a lazy event filter and use it to derive more analysis variables.
    auto filteredEvents = ptCutData.Filter("ptCutVector.size() > 2");   // String used to test jitting.
    auto filteredEventCount = filteredEvents.Count();
    auto ptCutMin = filteredEvents.Min(ptCutColumn);
    auto ptCutMax = filteredEvents.Max(ptCutColumn);

    auto defaultSettings = std::cout.flags();
    ANA_MSG_INFO("Total events: " << std::setprecision(16) << totalEvents.GetValue());
    ANA_MSG_INFO("Filtered events: " << filteredEventCount.GetValue());
    ANA_MSG_INFO("Range|{Electron.pt >= minPt}: " << "[" << ptCutMin.GetValue() << ", " << ptCutMax.GetValue() << "]");
    std::cout.flags(defaultSettings);

    return 0;
}
