#include "xAODdataSource/TxAODdataSource.h"
#include "AppUtils/TOptions.h"
#include "xAODRootAccess/Init.h"
#include "xAODBase/IParticleContainer.h"
#include "AsgTools/MessageCheck.h"
#include "ROOT/TDFInterface.hxx"
#include <iomanip>


using TDataSource = ROOT::Experimental::TDF::TDataSource;
using TH1DModel = ROOT::Experimental::TDF::TH1DModel;

int main(int argc, char* argv[]) {
    using namespace asg::msgUserCode;
    ANA_CHECK_SET_TYPE(int);   // Required since the return type is different than standard ATLAS StatusCode.

    AppUtils::TOptions options(argc, argv);
    options.interprete();
    if (options.isHelpRequested()) return 0;
    if (options.enableMT()) { ROOT::EnableImplicitMT(options.NThreads()); }

    ANA_CHECK(xAOD::Init()); // Set up the job for xAOD access.
    auto xdf = xAOD::DataSource::make_xAODdataFrame(options.fileNamePattern(), AppUtils::DEFAULT_TREE_NAME);

    // Extract vector variable from AuxContainer
    const auto readPt = [](const xAOD::IParticleContainer* particles) {
        std::vector<double> ptVector{};
        for (const auto& p: *particles) {
            ptVector.push_back(p->pt());
        }
        return ptVector;
    };

    constexpr auto ptColumnName = "pt";
    auto ptData = xdf.Define(ptColumnName, readPt, {"Electrons"});    //Define a new column
    const TH1DModel& histModel = {"histElectronPt", "histElectronPt", 100, 0.0, 1500.0};
    auto histElectronPt = ptData.Histo1D(histModel, ptColumnName);

    // Access data
    auto defaultSettings = std::cout.flags();
    ANA_MSG_INFO("Histogram Electorn.pt:");
    ANA_MSG_INFO("Total entries: " << std::setprecision(16) << histElectronPt->GetEntries());
    ANA_MSG_INFO("Range: [" << histElectronPt->GetMinimum() << ", " << histElectronPt->GetMaximum() << "]");
    ANA_MSG_INFO("Mean: " << histElectronPt->GetMean() << " StdDev: " << histElectronPt->GetStdDev());
    std::cout.flags(defaultSettings);

    return 0;
}
