#include "xAODdataSource/TxAODdataSource.h"
#include "AppUtils/TOptions.h"
#include "xAODEventInfo/EventInfo.h"
#include "xAODRootAccess/Init.h"
#include "AsgTools/MessageCheck.h"
#include <iomanip>

using TDataSource = ROOT::Experimental::TDF::TDataSource;
using EventInfo = xAOD::EventInfo;

int main(int argc, char* argv[]) {
    using namespace asg::msgUserCode;
    ANA_CHECK_SET_TYPE(int);   // Required since the return type is different than standard ATLAS StatusCode.

    AppUtils::TOptions options(argc, argv);
    options.interprete();
    if (options.isHelpRequested()) return 0;

    if (options.enableMT()) { ROOT::EnableImplicitMT(options.NThreads()); }

    ANA_CHECK(xAOD::Init()); // Set up the job for xAOD access.
    constexpr auto branchName = "EventInfo";
    constexpr auto runNumber = "RunNumber";
    constexpr auto eventNumber = "EventNumber";

    //Define transient columns mapping to xAOD functions EventInfo::runNumber(), EventInfo::eventNumber()
    auto xdf = xAOD::DataSource::make_xAODdataFrame(options.fileNamePattern(), AppUtils::DEFAULT_TREE_NAME);
    auto runNumberData = xdf.Define(runNumber, [](const EventInfo *ei) { return ei->runNumber(); }, {branchName});
    auto eventNumberData = xdf.Define(eventNumber, [](const EventInfo *ei) { return ei->eventNumber(); }, {branchName});

    //Apply lazy actions
    auto eventNumberMin = eventNumberData.Min({eventNumber});
    auto eventNumberMax = eventNumberData.Max({eventNumber});
    auto runNumberHist = runNumberData.Histo1D(runNumber);

    //Access data
    auto defaultSettings = std::cout.flags();
    ANA_MSG_INFO("EventInfo.eventNumber Range: [" << std::setprecision(16) << eventNumberMin.GetValue()
                                        << ", " << eventNumberMax.GetValue() << "]");

    ANA_MSG_INFO("Histogram EventInfo.runNumber:");
    ANA_MSG_INFO("Total entries: " << runNumberHist->GetEntries());
    ANA_MSG_INFO("Range: [" << runNumberHist->GetMinimum() << ", " << runNumberHist->GetMaximum() << "]");
    ANA_MSG_INFO("Mean: " << runNumberHist->GetMean() << " StdDev: " << runNumberHist->GetStdDev());
    std::cout.flags(defaultSettings);

    return 0;
}