#include "xAODdataSource/TEventCursor.h"
#include <iostream>
#include <algorithm>
#include "AsgTools/MessageCheck.h"


namespace xAOD {
namespace DataSource {
using namespace asg::msgUserCode;

TEventCursor::~TEventCursor() = default;

void TEventCursor::initialise(std::unique_ptr<TEvent> event) {
    m_event = std::move(event);
}

void TEventCursor::finalise() {
    m_event.reset(nullptr);
    m_columnHandles.clear();
}

void * TEventCursor::columnReader(std::string_view columnName, const std::type_info &typeInfo) {
    //Use existing handle if column is already available, add new otherwise.
    auto columnHandle = std::find_if(m_columnHandles.begin(),
                                     m_columnHandles.end(),
                                     [columnName](const std::unique_ptr<IHandle>& h){
                                         return h->columnName() == columnName;
                                     });

    if (columnHandle == m_columnHandles.end()){
        auto newItemStatusPair = m_columnHandles.emplace(makeHandle(columnName, typeInfo, m_event));
        columnHandle = newItemStatusPair.first;

        //Todo: Check error status if addition of handle fails. Ensure strong guarantee.
    }

    return (*columnHandle)->reader();
}

int TEventCursor::setEntry(const ULong64_t entry) {
    //std::cout << "Event[" << entry << "]\n";
    ANA_CHECK_SET_TYPE(int);
    m_event->getEntry(entry);
    for (auto &columnHandle: m_columnHandles) {
        ANA_CHECK(columnHandle->update());
    }

    return 0;
}

} // end namespace DataSource
} // end namespace xAOD