#ifndef XAODDATASOURCE_XAODDATASOURCE_H
#define XAODDATASOURCE_XAODDATASOURCE_H

#include "TEventCursor.h"

// AnalysisBase includes:
#include "xAODRootAccess/TEvent.h"

// ROOT includes:
#include "TObject.h"
#include "ROOT/TDataFrame.hxx"
#include "ROOT/TDataSource.hxx"


#include <map>

namespace xAOD {
namespace DataSource {

ROOT::Experimental::TDataFrame
make_xAODdataFrame(std::string_view fileNamePattern, std::string_view treeName);

std::unique_ptr<TChain>
makeDataChain(std::string_view fileNamePattern, std::string_view treeName);

class TxAODdataSource final : public ROOT::Experimental::TDF::TDataSource {

public:
    explicit TxAODdataSource(std::string_view fileNamePattern, std::string_view treeName);

    ~TxAODdataSource() override;

    void SetNSlots(unsigned int nSlots) override;

    void Initialise() override;

    void Finalise() override;

    bool HasColumn(std::string_view columnName) const override;

    const std::vector<std::string> &GetColumnNames() const override;

    std::string GetTypeName(std::string_view) const override;

    std::vector<std::pair<ULong64_t, ULong64_t>> GetEntryRanges() override;

    void SetEntry(unsigned int slot, ULong64_t entry) override;

    void InitSlot(unsigned int slot, ULong64_t firstEntry) override;

    void FinaliseSlot(unsigned int slot) override;

private:
    const std::string m_fileNamePattern;
    const std::string m_treeName;

    ULong64_t m_size{};
    std::unordered_map<std::string, std::string> m_columnClassMap{};
    std::vector<std::string> m_columnNames{};
    unsigned int m_NSlots{};
    std::vector<std::unique_ptr<TEventCursor>> m_eventCursors{};
    std::vector<std::pair<ULong64_t, ULong64_t>> m_entryRanges{};

    std::vector<void *> GetColumnReadersImpl(std::string_view columnName, const std::type_info &typeInfo) override;
    void InitMetaData();
    void TrimVersionSuffix(std::string &typeName);
};

} // name-space DataSource
} // name-space xAOD
#endif // XAODDATASOURCE_XAODDATASOURCE_H